@@ -907,6 +907,7 @@ notify_starting (void *cls)
     op->result_cont (op->cont_cls,
                      GNUNET_ARM_REQUEST_SENT_OK,
                      op->starting_ret);
+  GNUNET_DISK_file_close(op->rfd);
   GNUNET_free (op);
 }
 
@@ -977,9 +978,12 @@ GNUNET_ARM_request_service_start (struct GNUNET_ARM_Handle *h,
   {
     GNUNET_log_strerror (GNUNET_ERROR_TYPE_WARNING,
                          "pipe");
-  } else {
+    ret = GNUNET_ARM_RESULT_START_FAILED;
+  }
+  else
+  {
     wsig = GNUNET_DISK_pipe_detach_end (sig,
-                                      GNUNET_DISK_PIPE_END_WRITE);
+                                        GNUNET_DISK_PIPE_END_WRITE);
     ret = start_arm_service (h,
                              std_inheritance,
                              wsig);
