@@ -175,10 +175,15 @@ struct PeerResultMessage
    */
   uint32_t type GNUNET_PACKED;
 
+  /**
+   * Always 0.
+   */
+  uint16_t reserved GNUNET_PACKED;
+
   /**
    * Message options, actually an 'enum GNUNET_DHT_RouteOption' value in NBO.
    */
-  uint32_t options GNUNET_PACKED;
+  uint16_t options GNUNET_PACKED;
 
   /**
    * Length of the PUT path that follows (if tracked).
@@ -1771,7 +1776,8 @@ GDS_NEIGHBOURS_handle_reply (struct PeerInfo *pi,
     prm->header.type = htons (GNUNET_MESSAGE_TYPE_DHT_P2P_RESULT);
     prm->header.size = htons (sizeof (buf));
     prm->type = htonl ((uint32_t) bd->type);
-    prm->options = htonl ((uint32_t) ro);
+    prm->reserved = htons (0);
+    prm->options = htons ((uint16_t) ro);
     prm->put_path_length = htons ((uint16_t) ppl);
     prm->get_path_length = htons ((uint16_t) get_path_length);
     prm->expiration_time = GNUNET_TIME_absolute_hton (bd->expiration_time);
@@ -1959,8 +1965,8 @@ handle_dht_p2p_put (void *cls,
     : (const char *) &put_path[putlen];
   size_t var_meta_size
     = putlen * sizeof(struct GNUNET_DHT_PathElement)
-      + has_path ? sizeof (*last_sig) : 0
-      + truncated ? sizeof (*trunc_peer) : 0;
+      + (has_path ? sizeof (*last_sig) : 0)
+      + (truncated ? sizeof (*trunc_peer) : 0);
   struct GNUNET_DATACACHE_Block bd = {
     .key = put->key,
     .expiration_time = GNUNET_TIME_absolute_ntoh (put->expiration_time),
@@ -2044,7 +2050,7 @@ handle_dht_p2p_put (void *cls,
                      GNUNET_CONTAINER_bloomfilter_test (bf,
                                                         &peer->phash));
     /* extend 'put path' by sender */
-    bd.put_path = (const struct GNUNET_DHT_PathElement *) pp;
+    bd.put_path = pp;
     bd.put_path_length = putlen + 1;
     if (has_path)
     {
@@ -2078,7 +2084,7 @@ handle_dht_p2p_put (void *cls,
                     failure_offset);
         GNUNET_assert (failure_offset <= putlen + 1);
         bd.put_path = &pp[failure_offset];
-        bd.put_path_length = putlen - failure_offset;
+        bd.put_path_length = (putlen + 1) - failure_offset;
         bd.ro |= GNUNET_DHT_RO_TRUNCATED;
         bd.trunc_peer = pp[failure_offset - 1].pred;
       }
@@ -2473,9 +2479,10 @@ process_reply_with_path (const struct GNUNET_DATACACHE_Block *bd,
                                                         + bd->put_path_length)];
     struct GNUNET_DATACACHE_Block bdx = *bd;
 
-    GNUNET_memcpy (xput_path,
-                   bd->put_path,
-                   bd->put_path_length * sizeof(struct GNUNET_DHT_PathElement));
+    if (NULL != bd->put_path)
+      GNUNET_memcpy (xput_path,
+                     bd->put_path,
+                     bd->put_path_length * sizeof(struct GNUNET_DHT_PathElement));
     GNUNET_memcpy (&xput_path[bd->put_path_length],
                    get_path,
                    get_path_length * sizeof(struct GNUNET_DHT_PathElement));
@@ -2503,14 +2510,21 @@ static enum GNUNET_GenericReturnValue
 check_dht_p2p_result (void *cls,
                       const struct PeerResultMessage *prm)
 {
+  uint16_t msize = ntohs (prm->header.size) - sizeof (*prm);
+  enum GNUNET_DHT_RouteOption ro
+    = (enum GNUNET_DHT_RouteOption) ntohs (prm->options);
+  bool truncated = (0 != (ro & GNUNET_DHT_RO_TRUNCATED));
+  bool tracked = (0 != (ro & GNUNET_DHT_RO_RECORD_ROUTE));
+
   uint16_t get_path_length = ntohs (prm->get_path_length);
   uint16_t put_path_length = ntohs (prm->put_path_length);
-  uint16_t msize = ntohs (prm->header.size);
+  size_t vsize = (truncated ? sizeof (struct GNUNET_PeerIdentity) : 0)
+                 + (tracked ? sizeof (struct GNUNET_CRYPTO_EddsaSignature) : 0);
 
   (void) cls;
-  if ( (msize <
-        sizeof(struct PeerResultMessage)
-        + (get_path_length + put_path_length)
+  if ( (msize < vsize) ||
+       (msize - vsize <
+        (get_path_length + put_path_length)
         * sizeof(struct GNUNET_DHT_PathElement)) ||
        (get_path_length >
         GNUNET_MAX_MESSAGE_SIZE / sizeof(struct GNUNET_DHT_PathElement)) ||
@@ -2538,7 +2552,7 @@ handle_dht_p2p_result (void *cls,
   struct PeerInfo *peer = t->pi;
   uint16_t msize = ntohs (prm->header.size) - sizeof (*prm);
   enum GNUNET_DHT_RouteOption ro
-    = (enum GNUNET_DHT_RouteOption) ntohl (prm->options);
+    = (enum GNUNET_DHT_RouteOption) ntohs (prm->options);
   bool truncated = (0 != (ro & GNUNET_DHT_RO_TRUNCATED));
   bool tracked = (0 != (ro & GNUNET_DHT_RO_RECORD_ROUTE));
   uint16_t get_path_length = ntohs (prm->get_path_length);
