@@ -175,6 +175,12 @@ extern "C" {
 #define GNUNET_GNSRECORD_TYPE_MESSENGER_ROOM_DETAILS 65560
 
 
+/**
+ * Record type to store DID Documents
+ */
+#define GNUNET_GNSRECORD_TYPE_DID_DOCUMENT 65561
+
+
 
 #if 0 /* keep Emacsens' auto-indent happy */
 {
