@@ -19,6 +19,11 @@
  */
 
 /**
+ * @addtogroup Backbone
+ * @{
+ * @addtogroup dht_libs  DHT and support libraries
+ * @{
+ *
  * @author Christian Grothoff
  *
  * @file
@@ -138,7 +143,7 @@ struct GNUNET_DHT_HopSignature
   /**
    * Hash over the payload of the block.
    */
-  struct GNUNET_HashCode h_data;
+  struct GNUNET_HashCode h_data GNUNET_PACKED;
 
   /**
    * Previous hop the message was received from.  All zeros
@@ -580,3 +585,7 @@ GNUNET_DHT_hello_offer (struct GNUNET_DHT_Handle *dht_handle,
 #endif
 
 /** @} */  /* end of group dht */
+
+/** @} */ /* end of group addition */
+
+/** @} */ /* end of group addition */
