@@ -160,49 +160,36 @@ struct GNUNET_NAMESTORE_PluginFunctions
    * Start a transaction in the database
    *
    * @param cls closure (internal context for the plugin)
-   * @return #GNUNET_OK on success, #GNUNET_NO if there were no results, #GNUNET_SYSERR on error
+   * @param emsg message. On error, string will be allocated and must be freed.
+   * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
    */
   enum GNUNET_GenericReturnValue
-  (*transaction_begin) (void *cls);
+  (*transaction_begin) (void *cls, char **emsg);
 
   /**
-   * Abort a transaction in the database
+   * Abort and roll back a transaction in the database
    *
    * @param cls closure (internal context for the plugin)
-   * @return #GNUNET_OK on success, #GNUNET_NO if there were no results, #GNUNET_SYSERR on error
+   * @param emsg message. On error, string will be allocated and must be freed.
+   * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
    */
   enum GNUNET_GenericReturnValue
-  (*transaction_abort) (void *cls);
+  (*transaction_rollback) (void *cls, char **emsg);
 
   /**
    * Commit a transaction in the database
    *
    * @param cls closure (internal context for the plugin)
-   * @return #GNUNET_OK on success, #GNUNET_NO if there were no results, #GNUNET_SYSERR on error
+   * @param emsg message. On error, string will be allocated and must be freed.
+   * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
    */
   enum GNUNET_GenericReturnValue
-  (*transaction_commit) (void *cls);
+  (*transaction_commit) (void *cls, char **emsg);
 
   /**
-   * Replace a record in the datastore for which we are the authority.
-   * Removes any existing record in the same zone with the same name.
-   *
-   * @param cls closure (internal context for the plugin)
-   * @param zone private key of the zone
-   * @param label name of the record in the zone
-   * @param rd_count number of entries in @a rd array, 0 to delete all records
-   * @param rd array of records with data to store
-   * @return #GNUNET_OK on success, else #GNUNET_SYSERR
-   */
-  int
-  (*replace_records) (void *cls,
-                    const struct GNUNET_IDENTITY_PrivateKey *zone,
-                    const char *label,
-                    unsigned int rd_count,
-                    const struct GNUNET_GNSRECORD_Data *rd);
-
-  /**
-   * Lookup records in the datastore for which we are the authority.
+   * Edit records in the datastore for which we are the authority.
+   * Should be called within a transaction (after begin) and maps
+   * to a SELECT ... FOR UPDATE in PQ.
    *
    * @param cls closure (internal context for the plugin)
    * @param zone private key of the zone
@@ -212,7 +199,7 @@ struct GNUNET_NAMESTORE_PluginFunctions
    * @return #GNUNET_OK on success, #GNUNET_NO for no results, else #GNUNET_SYSERR
    */
   int
-  (*select_records) (void *cls,
+  (*edit_records) (void *cls,
                      const struct GNUNET_IDENTITY_PrivateKey *zone,
                      const char *label,
                      GNUNET_NAMESTORE_RecordIterator iter,
