@@ -145,6 +145,11 @@ struct LabelLookupMessage
    */
   uint32_t label_len GNUNET_PACKED;
 
+  /**
+   * GNUNET_YES if this lookup corresponds to an edit request
+   */
+  uint32_t is_edit_request GNUNET_PACKED;
+
   /**
    * The private key of the zone to look up in
    */
@@ -305,6 +310,54 @@ struct RecordResultMessage
    */
 };
 
+/**
+ * Send a transaction control message.
+ */
+struct TxControlMessage
+{
+  /**
+   * Type will be #GNUNET_MESSAGE_TYPE_NAMESTORE_TX_CONTROL
+   */
+  struct GNUNET_NAMESTORE_Header gns_header;
+
+  /**
+   * The type of control message to send
+   */
+  uint16_t control GNUNET_PACKED;
+
+  /**
+   * always zero (for alignment)
+   */
+  uint16_t reserved GNUNET_PACKED;
+
+};
+
+/**
+ * Result of a transaction control message.
+ */
+struct TxControlResultMessage
+{
+  /**
+   * Type will be #GNUNET_MESSAGE_TYPE_NAMESTORE_TX_CONTROL_RESULT
+   */
+  struct GNUNET_NAMESTORE_Header gns_header;
+
+  /**
+   * The type of control message to send
+   */
+  uint16_t control GNUNET_PACKED;
+
+  /**
+   * Of type GNUNET_GenericReturnValue
+   */
+  uint16_t success GNUNET_PACKED;
+
+  /* followed by:
+   * an error message if status != ntohs(GNUNET_OK)
+   */
+};
+
+
 
 /**
  * Start monitoring a zone.
