@@ -38,9 +38,9 @@ static struct GNUNET_NAMESTORE_Handle *nsh;
 
 static struct GNUNET_SCHEDULER_Task *endbadly_task;
 
-static struct GNUNET_CRYPTO_EcdsaPrivateKey privkey;
+static struct GNUNET_IDENTITY_PrivateKey privkey;
 
-static struct GNUNET_CRYPTO_EcdsaPrivateKey privkey2;
+static struct GNUNET_IDENTITY_PrivateKey privkey2;
 
 static struct GNUNET_NAMESTORE_ZoneMonitor *zm;
 
@@ -129,7 +129,7 @@ end (void *cls)
 
 static void
 zone_proc (void *cls,
-           const struct GNUNET_CRYPTO_EcdsaPrivateKey *zone_key,
+           const struct GNUNET_IDENTITY_PrivateKey *zone_key,
            const char *name,
            unsigned int rd_count,
            const struct GNUNET_GNSRECORD_Data *rd)
@@ -248,28 +248,6 @@ put_cont (void *cls,
     return;
   }
 
-  if (3 == c)
-  {
-    /* Start monitoring */
-    zm = GNUNET_NAMESTORE_zone_monitor_start (cfg,
-                                              &privkey,
-                                              GNUNET_YES,
-                                              &fail_cb,
-                                              NULL,
-                                              &zone_proc,
-                                              NULL,
-                                              &sync_cb,
-                                              NULL);
-    if (NULL == zm)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Failed to create zone monitor\n");
-      GNUNET_break (0);
-      res = 1;
-      GNUNET_SCHEDULER_shutdown ();
-      return;
-    }
-  }
 }
 
 
@@ -302,8 +280,10 @@ run (void *cls,
      struct GNUNET_TESTING_Peer *peer)
 {
   res = 1;
-  GNUNET_CRYPTO_ecdsa_key_create (&privkey);
-  GNUNET_CRYPTO_ecdsa_key_create (&privkey2);
+  privkey.type = htonl (GNUNET_GNSRECORD_TYPE_PKEY);
+  privkey2.type = htonl (GNUNET_GNSRECORD_TYPE_PKEY);
+  GNUNET_CRYPTO_ecdsa_key_create (&privkey.ecdsa_key);
+  GNUNET_CRYPTO_ecdsa_key_create (&privkey2.ecdsa_key);
 
   cfg = mycfg;
   GNUNET_SCHEDULER_add_shutdown (&end,
@@ -322,6 +302,25 @@ run (void *cls,
                                               NULL);
     return;
   }
+  /* Start monitoring */
+  zm = GNUNET_NAMESTORE_zone_monitor_start (cfg,
+                                            &privkey,
+                                            GNUNET_YES,
+                                            &fail_cb,
+                                            NULL,
+                                            &zone_proc,
+                                            NULL,
+                                            &sync_cb,
+                                            NULL);
+  if (NULL == zm)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to create zone monitor\n");
+    GNUNET_break (0);
+    res = 1;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Created record 3\n");
