@@ -96,10 +96,11 @@ init ()
   if (GNUNET_YES == initialized)
     return;
   initialized = GNUNET_YES;
-  GNUNET_PLUGIN_load_all ("libgnunet_plugin_reclaim_attribute_",
-                          NULL,
-                          &add_plugin,
-                          NULL);
+  GNUNET_PLUGIN_load_all_in_context (GNUNET_OS_project_data_default (),
+                                     "libgnunet_plugin_reclaim_attribute_",
+                                     NULL,
+                                     &add_plugin,
+                                     NULL);
 }
 
 /**
@@ -249,16 +250,6 @@ GNUNET_RECLAIM_attribute_value_to_string (uint32_t type,
 }
 
 
-/**
- * Create a new attribute.
- *
- * @param attr_name the attribute name
- * @param credential credential ID of the attribute (maybe NULL)
- * @param type the attribute type
- * @param data the attribute value
- * @param data_size the attribute value size
- * @return the new attribute
- */
 struct GNUNET_RECLAIM_Attribute *
 GNUNET_RECLAIM_attribute_new (const char *attr_name,
                               const struct
